mvqtlmap <-
function(inputfile,method="CIM",qtlmodel=1:5,nbmk=15,winsize=10,mapfuncid=1,ifpermu=FALSE,permutimes=1000,np=1)
{
  library(doParallel)
  # Reading data

  if(!file.exists(inputfile)){
	stop("Input data file does not exist!")
  }
  if(method!="IM" && method!="CIM"){
	stop("Please choose a correct mapping method!")
  }
  rd <- mvqtl_readdata(inputfile)
  nqtlms=length(qtlmodel)
  mkp1<-rd[[1]]
  mkp2<-rd[[2]]
  mp<-rd[[3]]
  mk<-rd[[4]]
  t<-rd[[5]]
  y<-rd[[6]]
  n<-rd[[7]]
  ng<-rd[[8]]
  ngi<-rd[[9]]
  nt<-rd[[10]]
  n1<-nrow(y);
  n2<-ncol(y);
  
  gdst <- c()
  gdst[1] <- 0
  for(i in 1:ng){
    gdst[i+1] <- gdst[i]+sum(mp[[i]])	
  }
  
  gmkdst <- mp
  for(i in 1:ng){
    gmkdst[[i]] <- c(0,gmkdst[[i]])
    for(j in 1:(ngi[i]-1)){
      gmkdst[[i]][j+1] <- gmkdst[[i]][j] + mp[[i]][j]
    }
  }
  
# Sampling missing data
#  print("Sampling missing markers...")
#  for(i in 1:ng)
#    for(j in 1:ngi[i])
#      for(k in 1:n)
#        if(mk[[i]][j,k]=="--"){
#          mk[[i]][j,k]=mvqtl_sampling_missing_marker(i,j,k,mk,mp,mkp1,mkp2,mapfuncid)
#        }

  if(method=="CIM"){  # being CIM mapping and permutations
  	# Stepwise regression
	  print("Performing stepwise regression ...")
	  regrst <- mvqtl_regstepwise(mk,mkp1,mkp2,y,n,ng,ngi,winsize,gmkdst) # adding winsize and gmkdst,may 5,2016  
	  mkord <- regrst[[1]]
	  mkx <- regrst[[2]]

	  nmkord <- nrow(mkord)
	  if(nbmk > nmkord){ nbmk <- nmkord - 1 }
	  
	  mkordtmp <- mkord
	  for(i in 1:nmkord){
		mkordtmp[i,3] <- formatC(mkord[i,3],6,format="f")
		mkordtmp[i,4] <- formatC(mkord[i,4],6,format="f")
	  }
	  
	  write.table(mkordtmp,"backmarkers.txt",quote = FALSE, row.names =FALSE, col.names=FALSE)

  	# QTL CIM mapping
	  print("Performing CIM QTL mapping ...")
  
	  prefile = unlist(strsplit(inputfile,'.',fixed=TRUE))[1]
	  prefile = paste(prefile,"_CIM",sep="")
	  qtlrstfiles <- c()
	  for(qtlmodeli in 1:nqtlms){
	  	outputfile = paste(prefile,"_M",qtlmodel[qtlmodeli],"Rst.txt",sep="")
	  	qtlrstfiles <- c(qtlrstfiles,outputfile)
	  	if(file.exists(outputfile)){
			file.remove(outputfile)
	  	}
  	  	for(lgi in 1:ng)
  	 		for(mki in 1:(ngi[lgi]-1)){
     			rst<-mvqtlcim_mapping(lgi,mki,mkp1,mkp2,mk,mp,y,t,qtlmodel[qtlmodeli],
                           nbmk,winsize,mkord,mkx,mapfuncid,gdst,gmkdst)
      			write.table(rst,outputfile,append=TRUE, quote = FALSE, row.names =FALSE, col.names=FALSE)
    	  	}
    	  }
  
	# Permutation
  	if(ifpermu){
		print("Performing CIM permutations ...")
		for(qtlmodeli in 1:nqtlms){
     		dirname <- paste(prefile,"_permu",qtlmodel[qtlmodeli],sep="")
    		if(!dir.exists(dirname)){
      			dir.create(dirname)
    		}
    		cls <- makeCluster(np)
    		registerDoParallel(cls)
    		foreach(lst=1:permutimes,.export=c("mvqtlcim_mapping","cm2r","mvqtl_conprb",
				"mvqtl_transprb","mvqtl_backmks","mvqtl_mnord") ) %dopar% {
				outpermfile <- paste(dirname,"/","permu",qtlmodel[qtlmodeli],"_",lst,".txt",sep="")
				if(file.exists(outpermfile)){
					file.remove(outpermfile)
				}
				yperm <- y[sample(n1),]
				for(lgi in 1:ng){
        			for(mki in 1:(ngi[lgi]-1)){
						rst<-mvqtlcim_mapping(lgi,mki,mkp1,mkp2,mk,mp,yperm,t,qtlmodel[qtlmodeli],
                            	nbmk,winsize,mkord,mkx,mapfuncid,gdst,gmkdst,permu=1);
						write.table(rst,outpermfile,append=TRUE, quote = FALSE, row.names =FALSE, col.names=FALSE)
        			}
				}
				return (1)
    		}
    		stopCluster(cls)
		}
	}  
  }## end of CIM mapping and permutations

 if(method=="IM"){ 					 # Begin to perform IM mapping and permutations
  	# QTL funcmapping
	  print("Performing interval QTL mapping ...") 
	  prefile = unlist(strsplit(inputfile,'.',fixed=TRUE))[1]
	  prefile = paste(prefile,"_IM",sep="")
	  qtlrstfiles <- c()
	  for(qtlmodeli in 1:nqtlms){
	  	outputfile = paste(prefile,"_M",qtlmodel[qtlmodeli],"Rst.txt",sep="")
	  	qtlrstfiles <- c(qtlrstfiles,outputfile)
	  	if(file.exists(outputfile)){
			file.remove(outputfile)
	  	}
  	  	for(lgi in 1:ng)
  	 		for(mki in 1:(ngi[lgi]-1)){
     			rst<-mvqtlim_mapping(lgi,mki,mkp1,mkp2,mk,mp,y,t,
					qtlmodel[qtlmodeli],mapfuncid,gdst,gmkdst)
				write.table(rst,outputfile,append=TRUE, quote = FALSE, row.names =FALSE, col.names=FALSE)
    	  	}
	  }

	# Permutation
  	if(ifpermu){
		print("Performing permutations ...")
		for(qtlmodeli in 1:nqtlms){
     		dirname <- paste(prefile,"_permu",qtlmodel[qtlmodeli],sep="")
    		if(!dir.exists(dirname)){
      			dir.create(dirname)
    		}
    		cls <- makeCluster(np)
    		registerDoParallel(cls)
    		foreach(lst=1:permutimes,.export=c("mvqtlim_mapping","cm2r","mvqtl_conprb",
				"mvqtl_transprb","mvqtl_mnord") ) %dopar% {
      			outpermfile <- paste(dirname,"/","permu",qtlmodel[qtlmodeli],"_",lst,".txt",sep="")
				if(file.exists(outpermfile)){
					file.remove(outpermfile)
				}
      			yperm <- y[sample(n1),]
      			for(lgi in 1:ng){
        				for(mki in 1:(ngi[lgi]-1)){
          				rst<-mvqtlim_mapping(lgi,mki,mkp1,mkp2,mk,mp,yperm,t,
							qtlmodel[qtlmodeli],mapfuncid,gdst,gmkdst,permu=1);
          				write.table(rst,outpermfile,append=TRUE, quote = FALSE, row.names =FALSE, col.names=FALSE)
        				}
      			}
      			return (1)
    		}
    		stopCluster(cls)
		}
	}

 }  ## end of IM mapping and permutations


  # dealing with the permutation files
  if(ifpermu){
	permrstfiles <- c()	
	for(i in 1:nqtlms){
		dirname <- paste(prefile,"_permu",qtlmodel[i],sep="")
	    if(!dir.exists(dirname)){
			stop(paste("Permutation did not finish for model", i))
    	}
		tmpfile <- paste(dirname,"/","permu",qtlmodel[i],"_1.txt",sep="")
	    if(!file.exists(tmpfile)){
		    stop(paste("Permutation did not finish for model", i))
    	}
		pmdat0 <- read.table(tmpfile)
		dmdat0 <- dim(pmdat0)
		pmdat <- pmdat0[,1:2]
		
		for(j in 2:permutimes){
			tmpfile <- paste(dirname,"/","permu",qtlmodel[i],"_",j,".txt",sep="")
	    	if(!file.exists(tmpfile)){
				stop(paste("Permutation did not finish for model", i))
    		}
			pmdat0 <- read.table(tmpfile)
			dmdat1 <- dim(pmdat0)
			if(dmdat1[1]!=dmdat0[1] || dmdat1[2]!=dmdat0[2]){
				stop(paste("Permutation did not finish for model", i))
			}
			pmdat <- cbind(pmdat,pmdat0[,2])
		}
		
		permufile <-	paste(prefile,"_PermuM",qtlmodel[i],"Rst.txt",sep="")
		permrstfiles <- c(permrstfiles,permufile)
		write.table(pmdat,permufile,quote = FALSE, row.names =FALSE, col.names=FALSE)
	}
  }
  
  #  Plotting LR profiles and summarizing QTL mapping results

  plottype=c("pdf","png","jpg","tif","bmp")
  if(ifpermu){
    print("Plotting LR profiles and summarizing QTL mapping results ...")
	for(i in 1:nqtlms){
		for(j in 1:5){
			mvqtlLRPlot(qtlrstfile=qtlrstfiles[i],permurstfile=permrstfiles[i],wglevel=1,type=plottype[j])
		}
	}
	for(i in 1:nqtlms){
		for(j in 1:5){
			mvqtlLRPlot(qtlrstfile=qtlrstfiles[i],permurstfile=permrstfiles[i],wglevel=2,type=plottype[j])
		}
	}
  }
}
